@@ -12,8 +12,11 @@ extern "C" {
 #pragma }
 #endif /* __cplusplus }*/
 
+#include <camel/camel-offline-folder.h>
 #include <camel/camel-folder.h>
 #include <camel/camel-data-cache.h>
+#include <camel/camel-offline-folder.h>
+#include <camel/camel-offline-journal.h>
 #include "camel-stub.h"
 
 #define CAMEL_EXCHANGE_FOLDER_TYPE     (camel_exchange_folder_get_type ())
@@ -22,17 +25,18 @@ extern "C" {
 #define CAMEL_IS_EXCHANGE_FOLDER(o)    (CAMEL_CHECK_TYPE((o), CAMEL_EXCHANGE_FOLDER_TYPE))
 
 typedef struct {
-	CamelFolder parent_object;
+	CamelOfflineFolder parent_object;
 
 	CamelStub *stub;
 	CamelDataCache *cache;
+	CamelOfflineJournal *journal;
 	char *source;
 
 	GHashTable *thread_index_to_message_id;
 } CamelExchangeFolder;
 
 typedef struct {
-	CamelFolderClass parent_class;
+	CamelOfflineFolderClass parent_class;
 
 } CamelExchangeFolderClass;
 
@@ -46,26 +50,27 @@ gboolean camel_exchange_folder_construct            (CamelFolder *folder,
 						     const char *name,
 						     guint32 camel_flags,
 						     const char *folder_dir,
+						     int offline_state,
 						     CamelStub *stub,
 						     CamelException *ex);
 
-void     camel_exchange_folder_add_message          (CamelExchangeFolder *,
+void     camel_exchange_folder_add_message          (CamelExchangeFolder *exch,
 						     const char *uid,
 						     guint32 flags,
 						     guint32 size,
 						     const char *headers);
 
-void     camel_exchange_folder_remove_message       (CamelExchangeFolder *,
+void     camel_exchange_folder_remove_message       (CamelExchangeFolder *exch,
 						     const char *uid);
 
-void     camel_exchange_folder_uncache_message      (CamelExchangeFolder *,
+void     camel_exchange_folder_uncache_message      (CamelExchangeFolder *exch,
 						     const char *uid);
 
-void     camel_exchange_folder_update_message_flags (CamelExchangeFolder *,
+void     camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
 						     const char *uid,
 						     guint32 flags);
 
-void     camel_exchange_folder_update_message_tag   (CamelExchangeFolder *,
+void     camel_exchange_folder_update_message_tag   (CamelExchangeFolder *exch,
 						     const char *uid,
 						     const char *name,
 						     const char *value);
