@@ -568,31 +568,30 @@ e2k_context_fba (E2kContext *ctx, SoupMessage *failed_msg)
 			continue;
 		value = xmlGetProp (node, "value");
 
-		if (form_body->len > 0)
-			g_string_append_c (form_body, '&');
-
 		if (!g_ascii_strcasecmp (name, "destination") && value) {
 			g_string_append (form_body, name);
 			g_string_append_c (form_body, '=');
-			e2k_uri_append_encoded (form_body, value, NULL);
+			e2k_uri_append_encoded (form_body, value, FALSE, NULL);
+			g_string_append_c (form_body, '&');
 		} else if (!g_ascii_strcasecmp (name, "flags")) {
 			g_string_append_printf (form_body, "flags=%d",
 						E2K_FBA_FLAG_TRUSTED);
+			g_string_append_c (form_body, '&');
 		} else if (!g_ascii_strcasecmp (name, "username")) {
 			g_string_append (form_body, "username=");
-			e2k_uri_append_encoded (form_body, ctx->priv->username, NULL);
+			e2k_uri_append_encoded (form_body, ctx->priv->username, FALSE, NULL);
+			g_string_append_c (form_body, '&');
 		} else if (!g_ascii_strcasecmp (name, "password")) {
 			g_string_append (form_body, "password=");
-			e2k_uri_append_encoded (form_body, ctx->priv->password, NULL);
-		} else if (!g_ascii_strcasecmp (name, "trusted")) {
-			g_string_append_printf (form_body, "trusted=%d",
-						E2K_FBA_FLAG_TRUSTED);
+			e2k_uri_append_encoded (form_body, ctx->priv->password, FALSE, NULL);
+			g_string_append_c (form_body, '&');
 		}
 
 		if (value)
 			xmlFree (value);
 		xmlFree (name);
 	}
+	g_string_append_printf (form_body, "trusted=%d", E2K_FBA_FLAG_TRUSTED);
 	xmlFreeDoc (doc);
 	doc = NULL;
 
@@ -728,10 +727,10 @@ setup_message (SoupMessageFilter *filter, SoupMessage *msg)
 /**
  * e2k_soup_message_new:
  * @ctx: the context
- * @uri: URI, as with soup_context_get()
- * @method: method, as with soup_message_new()
+ * @uri: the URI
+ * @method: the HTTP method
  *
- * Use this instead of soup_message_new().
+ * Creates a new %SoupMessage for @ctx.
  *
  * Return value: a new %SoupMessage, set up for connector use
  **/
@@ -753,14 +752,14 @@ e2k_soup_message_new (E2kContext *ctx, const char *uri, const char *method)
 /**
  * e2k_soup_message_new_full:
  * @ctx: the context
- * @uri: URI, as with soup_context_get()
- * @method: method, as with soup_message_new_full()
+ * @uri: the URI
+ * @method: the HTTP method
  * @content_type: MIME Content-Type of @body
  * @owner: ownership of @body
  * @body: request body
  * @length: length of @body
  *
- * Use this instead of soup_message_new_full().
+ * Creates a new %SoupMessage with the given body.
  *
  * Return value: a new %SoupMessage with a request body, set up for
  * connector use
@@ -787,8 +786,7 @@ e2k_soup_message_new_full (E2kContext *ctx, const char *uri,
  * @callback: callback to invoke when @msg is done
  * @user_data: data for @callback
  *
- * Asynchronously queues @msg. Use this instead of
- * soup_session_queue_message().
+ * Asynchronously queues @msg in @ctx's session.
  **/
 void
 e2k_context_queue_message (E2kContext *ctx, SoupMessage *msg,
@@ -817,8 +815,7 @@ context_canceller (E2kOperation *op, gpointer owner, gpointer data)
  * @op: an #E2kOperation to use for cancellation
  * @msg: the message to send
  *
- * Synchronously sends @msg. Use this instead of
- * soup_session_send_message().
+ * Synchronously sends @msg in @ctx's session.
  *
  * Return value: the HTTP status of the message
  **/
@@ -899,10 +896,11 @@ get_msg (E2kContext *ctx, const char *uri, gboolean owa, gboolean claim_ie)
  * @body: if not %NULL, will contain the response body on return
  * @len: if not %NULL, will contain the response body length on return
  *
- * Performs a GET on @ctx for @uri. If successful, the Content-Type,
- * body and length will be returned. The body is not terminated by a
- * '\0'. If the GET is not successful, @content_type, @body and @len
- * will be untouched, even if the server's response included a body.
+ * Performs a GET on @ctx for @uri. If successful (2xx status code),
+ * the Content-Type, body and length will be returned. The body is not
+ * terminated by a '\0'. If the GET is not successful, @content_type,
+ * @body and @len will be untouched (even if the error response
+ * included a body).
  *
  * Return value: the HTTP status
  **/
@@ -1076,9 +1074,9 @@ e2k_context_put (E2kContext *ctx, E2kOperation *op, const char *uri,
  * @content_type: MIME Content-Type of the data
  * @body: data to PUT
  * @length: length of @body
- * @location: if not %NULL, will contain the Location of the POSTed
+ * @location: if not %NULL, will contain the Location of the PUT
  * object on return
- * @repl_uid: if not %NULL, will contain the Repl-UID of the POSTed
+ * @repl_uid: if not %NULL, will contain the Repl-UID of the PUT
  * object on return
  *
  * PUTs data into @folder_uri on @ctx with a new name based on
@@ -1108,7 +1106,7 @@ e2k_context_put_new (E2kContext *ctx, E2kOperation *op,
 	g_return_val_if_fail (body != NULL, E2K_HTTP_MALFORMED);
 
 	slash_uri = e2k_strdup_with_trailing_slash (folder_uri);
-	encoded_name = e2k_uri_encode (object_name, NULL);
+	encoded_name = e2k_uri_encode (object_name, TRUE, NULL);
 
 	/* folder_uri is a dummy here */
 	msg = put_msg (ctx, folder_uri, content_type,
@@ -1196,6 +1194,8 @@ write_prop (GString *xml, const char *propertyname,
 	GPtrArray *array;
 	int i;
 
+	if (value == NULL)
+		return;
 	namespace = e2k_prop_namespace_name (propertyname);
 	abbrev = e2k_prop_namespace_abbrev (propertyname);
 	name = e2k_prop_property_name (propertyname);
@@ -1452,7 +1452,7 @@ e2k_context_proppatch_new (E2kContext *ctx, E2kOperation *op,
 	g_return_val_if_fail (props != NULL, E2K_HTTP_MALFORMED);
 
 	slash_uri = e2k_strdup_with_trailing_slash (folder_uri);
-	encoded_name = e2k_uri_encode (object_name, NULL);
+	encoded_name = e2k_uri_encode (object_name, TRUE, NULL);
 
 	/* folder_uri is a dummy here */
 	msg = patch_msg (ctx, folder_uri, "PROPPATCH", NULL, 0, props, TRUE);
@@ -1512,7 +1512,7 @@ bproppatch_free (E2kResultIter *iter, gpointer msg)
  * @props: the properties to set/remove
  * @create: whether or not to create @uri if it does not exist
  *
- * Begins BPROPPATCHing @hrefs based at @uri.
+ * Begins a BPROPPATCH (bulk PROPPATCH) of @hrefs based at @uri.
  *
  * Return value: an iterator for getting the results of the BPROPPATCH
  **/
@@ -1955,8 +1955,7 @@ bdelete_msg (E2kContext *ctx, const char *uri, const char **hrefs, int nhrefs)
 	GString *xml;
 	int i;
 
-	xml = g_string_new ("<?xml version=\"1.0\" encoding=\"utf-8\" ?>"
-			    "<delete xmlns=\"DAV:\"><target>");
+	xml = g_string_new (E2K_XML_HEADER "<delete xmlns=\"DAV:\"><target>");
 
 	for (i = 0; i < nhrefs; i++) {
 		g_string_append (xml, "<href>");
@@ -2099,6 +2098,40 @@ e2k_context_mkcol (E2kContext *ctx, E2kOperation *op,
 
 /* BMOVE / BCOPY */
 
+static SoupMessage *
+transfer_msg (E2kContext *ctx,
+	      const char *source_uri, const char *dest_uri,
+	      const char **source_hrefs, int nhrefs,
+	      gboolean delete_originals)
+{
+	SoupMessage *msg;
+	GString *xml;
+	int i;
+
+	xml = g_string_new (E2K_XML_HEADER);
+	g_string_append (xml, delete_originals ? "<move" : "<copy");
+	g_string_append (xml, " xmlns=\"DAV:\"><target>");
+	for (i = 0; i < nhrefs; i++) {
+		g_string_append (xml, "<href>");
+		e2k_g_string_append_xml_escaped (xml, source_hrefs[i]);
+		g_string_append (xml, "</href>");
+	}
+	g_string_append (xml, "</target></");
+	g_string_append (xml, delete_originals ? "move>" : "copy>");
+
+	msg = e2k_soup_message_new_full (ctx, source_uri,
+					 delete_originals ? "BMOVE" : "BCOPY",
+					 "text/xml",
+					 SOUP_BUFFER_SYSTEM_OWNED,
+					 xml->str, xml->len);
+	soup_message_add_header (msg->request_headers, "Overwrite", "f");
+	soup_message_add_header (msg->request_headers, "Allow-Rename", "t");
+	soup_message_add_header (msg->request_headers, "Destination", dest_uri);
+	g_string_free (xml, FALSE);
+
+	return msg;
+}
+
 static E2kHTTPStatus
 transfer_next (E2kResultIter *iter,
 	       E2kContext *ctx, E2kOperation *op,
@@ -2106,24 +2139,33 @@ transfer_next (E2kResultIter *iter,
 	       int *first, int *total,
 	       gpointer user_data)
 {
-	SoupMessage *msg = user_data;
+	GSList **msgs = user_data;
+	SoupMessage *msg;
 	E2kHTTPStatus status;
 
-	if (msg->status != SOUP_MESSAGE_STATUS_IDLE)
+	if (!*msgs)
 		return E2K_HTTP_OK;
 
+	msg = (*msgs)->data;
+	*msgs = g_slist_remove (*msgs, msg);
+
 	status = e2k_context_send_message (ctx, op, msg);
-	if (status == E2K_HTTP_MULTI_STATUS) {
+	if (status == E2K_HTTP_MULTI_STATUS)
 		e2k_results_from_multistatus (msg, results, nresults);
-		*total = *nresults;
-	}
+
+	g_object_unref (msg);
 	return status;
 }
 
 static void
-transfer_free (E2kResultIter *iter, gpointer msg)
+transfer_free (E2kResultIter *iter, gpointer user_data)
 {
-	g_object_unref (msg);
+	GSList **msgs = user_data, *m;
+
+	for (m = *msgs; m; m = m->next)
+		g_object_unref (m->data);
+	g_slist_free (*msgs);
+	g_free (msgs);
 }
 
 /**
@@ -2151,9 +2193,10 @@ e2k_context_transfer_start (E2kContext *ctx, E2kOperation *op,
 			    const char *source_folder, const char *dest_folder,
 			    GPtrArray *source_hrefs, gboolean delete_originals)
 {
+	GSList **msgs;
 	SoupMessage *msg;
 	char *dest_uri;
-	GString *xml;
+	const char **hrefs;
 	int i;
 
 	g_return_val_if_fail (E2K_IS_CONTEXT (ctx), NULL);
@@ -2162,32 +2205,20 @@ e2k_context_transfer_start (E2kContext *ctx, E2kOperation *op,
 	g_return_val_if_fail (source_hrefs != NULL, NULL);
 
 	dest_uri = e2k_strdup_with_trailing_slash (dest_folder);
+	hrefs = (const char **)source_hrefs->pdata;
 
-	xml = g_string_new ("<?xml version=\"1.0\" encoding=\"utf-8\" ?>");
-	g_string_append (xml, delete_originals ? "<move" : "<copy");
-	g_string_append (xml, " xmlns=\"DAV:\"><target>");
-	for (i = 0; i < source_hrefs->len; i++) {
-		g_string_append (xml, "<href>");
-		e2k_g_string_append_xml_escaped (xml, source_hrefs->pdata[i]);
-		g_string_append (xml, "</href>");
+	msgs = g_new0 (GSList *, 1);
+	for (i = 0; i < source_hrefs->len; i += E2K_CONTEXT_MAX_BATCH_SIZE) {
+		msg = transfer_msg (ctx, source_folder, dest_uri,
+				    hrefs + i, MIN (E2K_CONTEXT_MAX_BATCH_SIZE, source_hrefs->len - i),
+				    delete_originals);
+		*msgs = g_slist_append (*msgs, msg);
 	}
-	g_string_append (xml, "</target></");
-	g_string_append (xml, delete_originals ? "move>" : "copy>");
-
-	msg = e2k_soup_message_new_full (ctx, source_folder,
-					 delete_originals ? "BMOVE" : "BCOPY",
-					 "text/xml",
-					 SOUP_BUFFER_SYSTEM_OWNED,
-					 xml->str, xml->len);
-	soup_message_add_header (msg->request_headers, "Overwrite", "f");
-	soup_message_add_header (msg->request_headers, "Allow-Rename", "t");
-	soup_message_add_header (msg->request_headers, "Destination", dest_uri);
-	g_string_free (xml, FALSE);
 	g_free (dest_uri);
 
-	return e2k_result_iter_new (ctx, op, TRUE, -1,
+	return e2k_result_iter_new (ctx, op, TRUE, source_hrefs->len,
 				    transfer_next, transfer_free,
-				    msg); 
+				    msgs);
 }
 
 /**
