@@ -37,8 +37,8 @@
 #include "exchange-config-listener.h"
 #include "exchange-folder-size.h"
 
-#include <e-util/e-passwords.h>
-#include <e-util/e-path.h>
+#include <libedataserverui/e-passwords.h>
+#include "e2k-path.h"
 #include <libedataserver/e-source-list.h>
 
 #include <stdlib.h>
@@ -62,7 +62,8 @@ static void finalize (GObject *object);
 static gboolean is_empty (ExchangeHierarchy *hier);
 static void rescan (ExchangeHierarchy *hier);
 static ExchangeAccountFolderResult scan_subtree  (ExchangeHierarchy *hier,
-						  EFolder *folder);
+						  EFolder *folder,
+						  gboolean offline);
 static ExchangeAccountFolderResult create_folder (ExchangeHierarchy *hier,
 						  EFolder *parent,
 						  const char *name,
@@ -186,6 +187,13 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 	g_return_if_fail (E_IS_FOLDER (folder));
 	internal_uri = e_folder_exchange_get_internal_uri (folder);
 
+	/* This should ideally not be needed. But, this causes a problem when the
+	server has identical folder names [ internal_uri ] for folders. Very much
+	possible in the case of favorite folders */
+	if (g_hash_table_lookup (EXCHANGE_HIERARCHY_WEBDAV (hier)->priv->folders_by_internal_path,
+				(char *)e2k_uri_path (internal_uri)))
+		return;
+
 	g_hash_table_insert (EXCHANGE_HIERARCHY_WEBDAV (hier)->priv->folders_by_internal_path,
 			     (char *)e2k_uri_path (internal_uri), folder);
 
@@ -289,12 +297,11 @@ create_folder (ExchangeHierarchy *hier, EFolder *parent,
 	E2kProperties *props;
 	E2kHTTPStatus status;
 	char *permanent_url = NULL;
-	int i;
+	int i, mode;
 
-#ifdef OFFLINE_SUPPORT
-	if (exchange_account_is_offline (hier->account))
-		return EXCHANGE_ACCOUNT_FOLDER_OFFLINE;
-#endif
+	exchange_account_is_offline (hier->account, &mode);
+        if (mode != ONLINE_MODE)
+                return EXCHANGE_ACCOUNT_FOLDER_OFFLINE;
 
 	for (i = 0; folder_types[i].component; i++) {
 		if (!strcmp (folder_types[i].component, type))
@@ -338,11 +345,12 @@ static ExchangeAccountFolderResult
 remove_folder (ExchangeHierarchy *hier, EFolder *folder)
 {
 	E2kHTTPStatus status;
+	int mode;
 
-#ifdef OFFLINE_SUPPORT
-	if (exchange_account_is_offline (hier->account))
-		return EXCHANGE_ACCOUNT_FOLDER_OFFLINE;
-#endif
+        exchange_account_is_offline (hier->account, &mode);
+
+        if (mode != ONLINE_MODE)
+                return EXCHANGE_ACCOUNT_FOLDER_OFFLINE; 
 
 	if (folder == hier->toplevel)
 		return EXCHANGE_ACCOUNT_FOLDER_PERMISSION_DENIED;
@@ -363,17 +371,14 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 	E2kHTTPStatus status;
 	EFolder *dest;
 	char *permanent_url = NULL, *physical_uri;
-	char *conf_key_cal="/apps/evolution/calendar/sources";
-	char *conf_key_tasks="/apps/evolution/tasks/sources";
-	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type = NULL;
 	ExchangeAccountFolderResult ret_code;
-
-#ifdef OFFLINE_SUPPORT
-	if (exchange_account_is_offline (hier->account))
-		return EXCHANGE_ACCOUNT_FOLDER_OFFLINE;
-#endif
+	int offline;
+	
+	exchange_account_is_offline (hier->account, &offline);
+        if (offline != ONLINE_MODE)
+                return EXCHANGE_ACCOUNT_FOLDER_OFFLINE;
 
 	if (source == hier->toplevel)
 		return EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR;
@@ -395,7 +400,7 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 		if (remove_source)
 			exchange_hierarchy_removed_folder (hier, source);
 		exchange_hierarchy_new_folder (hier, dest);
-		scan_subtree (hier, dest);
+		scan_subtree (hier, dest, (offline == OFFLINE_MODE));
 		physical_uri = (char *) e_folder_get_physical_uri (source);
 		g_object_unref (dest);
 		ret_code = EXCHANGE_ACCOUNT_FOLDER_OK;
@@ -421,10 +426,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 		    (strcmp (folder_type, "calendar/public") == 0)) {
 			cal_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_cal);
-			remove_esource (hier->account, conf_key_cal,
+						CONF_KEY_CAL);
+			remove_esource (hier->account, EXCHANGE_CALENDAR_FOLDER,
 					physical_uri, &cal_source_list,
-					FALSE, FALSE);
+					FALSE);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 		}
@@ -432,10 +437,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 			 (strcmp (folder_type, "tasks/public") == 0)){
 			task_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_tasks);
-			remove_esource (hier->account, conf_key_tasks,
+						CONF_KEY_TASKS);
+			remove_esource (hier->account, EXCHANGE_TASKS_FOLDER,
 					physical_uri, &task_source_list,
-					FALSE, FALSE);
+					FALSE);
 			e_source_list_sync (task_source_list, NULL);
 			g_object_unref (task_source_list);
 		}
@@ -443,10 +448,10 @@ xfer_folder (ExchangeHierarchy *hier, EFolder *source,
 			 (strcmp (folder_type, "contacts/public") == 0)) {
 			cont_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						conf_key_contacts);
-			remove_esource (hier->account, conf_key_contacts,
+						CONF_KEY_CONTACTS);
+			remove_esource (hier->account, EXCHANGE_CONTACTS_FOLDER,
 					physical_uri, &cont_source_list, 
-					FALSE, TRUE);
+					FALSE);
 			e_source_list_sync (cont_source_list, NULL);
 			g_object_unref (cont_source_list);
 		}
@@ -472,22 +477,29 @@ add_href (gpointer path, gpointer folder, gpointer hrefs)
 	g_ptr_array_add (hrefs, path);
 }
 
+static const char *rescan_props[] = {
+	E2K_PR_EXCHANGE_FOLDER_SIZE,
+	E2K_PR_HTTPMAIL_UNREAD_COUNT
+};
+static const int n_rescan_props = sizeof (rescan_props) / sizeof (rescan_props[0]);
+
 static void
 rescan (ExchangeHierarchy *hier)
 {
 	ExchangeHierarchyWebDAV *hwd = EXCHANGE_HIERARCHY_WEBDAV (hier);
 	const char *prop = E2K_PR_HTTPMAIL_UNREAD_COUNT;
+	const char *folder_size, *folder_name;
 	GPtrArray *hrefs;
 	E2kResultIter *iter;
 	E2kResult *result;
 	EFolder *folder;
-	int unread;
+	int unread, offline;
+	gboolean personal = ( hier->type == EXCHANGE_HIERARCHY_PERSONAL );
+	gdouble fsize_d;
 
-	if (
-#ifdef OFFLINE_SUPPORT
-	    exchange_account_is_offline (hier->account) ||
-#endif
-	    hier->type == EXCHANGE_HIERARCHY_PUBLIC)
+	exchange_account_is_offline (hier->account, &offline);
+	if ( (offline != ONLINE_MODE) ||
+		hier->type == EXCHANGE_HIERARCHY_PUBLIC)
 		return;
 
 	hrefs = g_ptr_array_new ();
@@ -502,7 +514,7 @@ rescan (ExchangeHierarchy *hier)
 	iter = e_folder_exchange_bpropfind_start (hier->toplevel, NULL,
 						  (const char **)hrefs->pdata,
 						  hrefs->len,
-						  &prop, 1);
+						  rescan_props, n_rescan_props);
 	g_ptr_array_free (hrefs, TRUE);
 
 	while ((result = e2k_result_iter_next (iter))) {
@@ -519,6 +531,17 @@ rescan (ExchangeHierarchy *hier)
 
 		if (unread != e_folder_get_unread_count (folder))
 			e_folder_set_unread_count (folder, unread);
+
+		folder_size = e2k_properties_get_prop (result->props,
+						E2K_PR_EXCHANGE_FOLDER_SIZE);
+		if (folder_size && personal) {
+			if (e_folder_exchange_get_permanent_uri (folder)) {
+				folder_name = e_folder_get_name (folder);
+				fsize_d = g_ascii_strtod (folder_size, NULL)/1024;
+				exchange_folder_size_update (hwd->priv->foldersize, 
+							folder_name, fsize_d);
+			}
+		}
 	}
 	e2k_result_iter_free (iter);
 	g_object_unref (hier);
@@ -540,6 +563,8 @@ exchange_hierarchy_webdav_status_to_folder_result (E2kHTTPStatus status)
 ExchangeFolderSize *
 exchange_hierarchy_webdav_get_folder_size (ExchangeHierarchyWebDAV *hwd)
 {
+	g_return_val_if_fail (EXCHANGE_IS_HIERARCHY_WEBDAV (hwd), NULL);
+
 	return hwd->priv->foldersize;
 }
 
@@ -553,6 +578,7 @@ exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 	const char *name, *prop, *outlook_class, *permanenturl, *folder_size;
 	int unread;
 	gboolean hassubs;
+	gdouble fsize_d;
 
 	g_return_val_if_fail (EXCHANGE_IS_HIERARCHY_WEBDAV (hwd), NULL);
 	g_return_val_if_fail (E_IS_FOLDER (parent), NULL);
@@ -608,12 +634,22 @@ exchange_hierarchy_webdav_parse_folder (ExchangeHierarchyWebDAV *hwd,
 		e_folder_exchange_set_has_subfolders (folder, TRUE);
 	if (permanenturl) {
 		e_folder_exchange_set_permanent_uri (folder, permanenturl);
-		exchange_folder_size_update (hwd->priv->foldersize, permanenturl, name, folder_size);
+		/* FIXME : Find a better way of doing this */
+		fsize_d = g_ascii_strtod (folder_size, NULL)/1024 ;
+		exchange_folder_size_update (hwd->priv->foldersize, 
+						name, fsize_d);
 	}
 
 	return folder;
 }
 
+static void
+add_folders (ExchangeHierarchy *hier, EFolder *folder, gpointer folders)
+{
+	g_object_ref (folder);
+	g_ptr_array_add (folders, folder);
+}
+
 static const char *folder_props[] = {
 	E2K_PR_EXCHANGE_FOLDER_CLASS,
 	E2K_PR_HTTPMAIL_UNREAD_COUNT,
@@ -625,7 +661,7 @@ static const char *folder_props[] = {
 static const int n_folder_props = sizeof (folder_props) / sizeof (folder_props[0]);
 
 static ExchangeAccountFolderResult
-scan_subtree (ExchangeHierarchy *hier, EFolder *parent)
+scan_subtree (ExchangeHierarchy *hier, EFolder *parent, gboolean offline)
 {
 	static E2kRestriction *folders_rn;
 	ExchangeHierarchyWebDAV *hwd = EXCHANGE_HIERARCHY_WEBDAV (hier);
@@ -633,7 +669,19 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *parent)
 	E2kResultIter *iter;
 	E2kResult *result;
 	E2kHTTPStatus status;
-	EFolder *folder;
+	EFolder *folder, *tmp;
+	GPtrArray *folders;
+	int i;
+	
+	if (offline) {
+		folders = g_ptr_array_new ();
+		exchange_hierarchy_webdav_offline_scan_subtree (EXCHANGE_HIERARCHY (hier), add_folders, folders);
+		for (i = 0; i <folders->len; i++) {
+			tmp = (EFolder *)folders->pdata[i];
+			exchange_hierarchy_new_folder (hier, (EFolder *)folders->pdata[i]);
+		}
+		return EXCHANGE_ACCOUNT_FOLDER_OK;
+	}
 
 	if (!folders_rn) {
 		folders_rn =
@@ -659,14 +707,13 @@ scan_subtree (ExchangeHierarchy *hier, EFolder *parent)
 			subtrees = g_slist_prepend (subtrees, folder);
 		}
 		exchange_hierarchy_new_folder (hier, folder);
-		g_object_unref (folder);
 	}
 	status = e2k_result_iter_free (iter);
 
 	while (subtrees) {
 		folder = subtrees->data;
 		subtrees = g_slist_remove (subtrees, folder);
-		scan_subtree (hier, folder);
+		scan_subtree (hier, folder, offline);
 	}
 
 	return exchange_hierarchy_webdav_status_to_folder_result (status);
@@ -691,6 +738,7 @@ scan_offline_cb (const char *physical_path, const char *path, gpointer data)
 	folder = e_folder_exchange_new_from_file (sod->hier, mf_name);
 	if (!folder) {
 		unlink (mf_name);
+		g_free (mf_name);
 		if (!sod->badpaths)
 			sod->badpaths = g_ptr_array_new ();
 		g_ptr_array_add (sod->badpaths, g_strdup (path));
